#include <TimerOne.h>
#define filterSamples 10
#define pinTempomatState 11
#define pinEncoder 49
#define usSamplingTime 100000
volatile unsigned long int filterBuffer[filterSamples];

void initializeHAL(void)
{
	TCCR4A=0x00;
	TCCR4B=0xC2;
	TIMSK4=0x20;
	pinMode(pinEncoder,INPUT);
	pinMode(pinTempomatState,INPUT);
	Serial3.begin(9600);
}

bool getTempomatState(void)
{
	return digitalRead(pinTempomatState);
}

void scheduleController(void (*isr)())
{
  Timer1.initialize(usSamplingTime);
  Timer1.attachInterrupt(isr);
}

ISR(TIMER4_CAPT_vect)
{
  static byte ind=0;
  filterBuffer[ind]=(TCNT4>>1)+2;
  TCNT4=0;
  ind=(ind==filterSamples-1)?0:(ind+1);  
}

double getSpeed(void)
{
  double avg=0;
  for (byte i=0;i<filterSamples;i++)
  {
    avg+=(double)filterBuffer[i]/((double)filterSamples);
  }
  avg=(double)25000/avg;
  return (avg<1)?(0):(avg);
}

bool sendThrottle(double throttle)
{
	if (throttle<0 || throttle>1)
	{
		return false;
	}
	else
	{
		byte throttleVal=(byte)floor(throttle*255);
		Serial3.write(throttleVal);
		return true;
	}
}